"""
Overview
========
The data guessing module was created so that functions in in the dock widget would make somewhat intelligent
guesses at what the user might want to use ahead of time. This functionality is negelcted if the user wants it to
be, however, even guesses it does make are overwritable.  This is purely meant as a time saver for users.
"""

import PyQt4.QtGui

import re

# Note the `r` before any string containing '\b', this is
# because python interprets '\b' to be a backspace, but
# the re module takes it to mean a word-break.  The `r`
# signifies the string as being raw characters.

def GenerateRegExpFromName (name):
	"""
	Overview
	========
	This function will take the name provided and produce a regular expression that accepts
	strings very similar to the name provided.  The regular expression may start with a number,
	underscore, or nothing.  Then it must contain the name provided, in any mixture of capital and
	lower case letters.  Finally it may end with a number or underscore, or nothing of course.

	@param name: The name which you want turned into a regular expression.
	@type name: string

	@return: _sre.SRE_Pattern, a compiled regular expression retrieved from the I{re.compile ()} method. 
	"""

	# change each letter of the work into the form [Xx]
	# to support any combination of lower/upper case letters
	nameInv = name.swapcase ()
	nameFinal = ""
	for index in xrange (len (name)):
		nameFinal += "[" + name[index] + nameInv[index] + "]"

	# and then return it's regular expression surrounded with
	# non-letters and word-breaks, also it MUST be the beginning
	# of the expression to match
	return re.compile (r"^(\b|[0-9_])" + nameFinal + r"(\b|[0-9_])")

def GenerateRegExpFromWord (word):
	"""
	Overview
	========
	This function is currently the exact same as L{GenerateRegExpFromName<GenerateRegExpFromName>}.  Originally
	there was going to be a distinction in the types of regular expression objects returned.  As of now, the
	appropriate dock widgets still make use of this method, so if a compiled regular expression is ever desired,
	only this function must be changed.

	@param word: The word which you want turned into a regular expression.
	@type word: string

	@return: _sre.SRE_Pattern, a compiled regular expression retrieved from the I{re.compile ()} method. 
	"""

	# change each letter of the work into the form [Xx]
	# to support any combination of lower/upper case letters
	wordInv = word.swapcase ()
	wordFinal = ""
	for index in xrange (len (word)):
		wordFinal += "[" + word[index] + wordInv[index] + "]"

	# and then return it's regular expression surrounded with
	# non-letters and word-breaks
	return re.compile (r"(\b|[0-9_])" + wordFinal + r"(\b|[0-9_])")

def GuessAttributes (object, guessList):
	"""
	Overview
	========
	This function takes a DockWidget and a list of regular expressions.  It will run down the Basin Object Tree,
	attempting to add any object that fits into the regular expression.  When that occurs, it starts at the top
	the Basin Object Tree again and does the same for the second regular expression until all of the regular
	expressions have been tried.  Normal rules apply for adding objects to a Dock Widget's Holder.  This means
	that if the first two regular expressions find objects with different parents lists, the second will kick
	the first one out.

	@param object: This is the dock widget for which the guesses are being made on. 
	@type object: L{DockWidget<DockWidget>}

	@param guessList: A list of all the compiled regular expressions for which guesses are being made
	@type guessList: list <_sre.SRE_Pattern>
	"""

	# If you are guessing Attributes, clear out the dock first.
	object.clear ()
	if ((not object._bridge._connected) and (not object._bridge._parallel)) or (not object._bridge._autoGuess):
		return

	# Define an iterator across the existing items in the region viewer
	iter = PyQt4.QtGui.QTreeWidgetItemIterator (object._bridge._regionViewer.top)
	iter += 1

	while iter.value ():
		# When you have no more guesses to make, return
		if len (guessList) == 0:
			return
		# For each regular expression hit
		if guessList[0].search (str (iter.value ().text (0))):
			# Remove the current regular expression you were
			# searching for
			guessList.pop (0)
			# Add the item in the region viewer that matched
			#### Think about checking the list in the object
			#### so that you don't have conflicting list parents
			object.DoubleClicked (iter.value ())
			# And reset your iterator to start at the top again
			iter = PyQt4.QtGui.QTreeWidgetItemIterator (object._bridge._regionViewer.top)
		# Progress to the next item, if you just matched then
		# the first item (the `BASIN` at the top) will never match
		# anyway, so it's ok to skip it
		iter += 1
