import PyQt4.QtCore
import PyQt4.QtGui

import Constants
import DataGuesser
import Widget
import Holders

class AbstractFunction (Widget.DockWidget):
	def __init__ (self, parent, bridge):
		self._type = "General Function"
		Widget.DockWidget.__init__ (self, parent, bridge)

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout ()
		box.setLayout (layout)

		executeButton = Constants.BQPushButton ("Execute", layout)
		PyQt4.QtCore.QObject.connect (executeButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.Execute)

		clearButton = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (clearButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def _addDataSelection (self):
		self._in = AttributeHolder (self._bridge, self)
		self._out = NewAttributeHolder (self._defaultOutName)

		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		self.rightLayout = PyQt4.QtGui.QGridLayout ()
		self.rightLayout.addWidget (PyQt4.QtGui.QLabel ("in:"), 0, 0)
		self.rightLayout.addWidget (self._in, 0 ,1)
		self.rightLayout.addWidget (PyQt4.QtGui.QLabel ("out:"), 0, 2)
		self.rightLayout.addWidget (self._out, 0, 3)
		outline.setLayout (self.rightLayout)

		return outline

	def Execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		commands = []
		commands.append (self._out.text () + " = " + self._functionName + " (" + self._in._shellAccess + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		if attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and attr._type == "_basin.Attribute"):
			self._in.setText (attr._name)
			self._in._name = attr._name
			self._in._shellAccess = attr._shellAccess

	def clear (self):
		self._in.clear ()
		self._out.clear ()
		return


class Sort (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "sort"
		self._displayName = "Sort"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/sort.png")
		self._description = "This function takes an Attribute in, sorts it, then sets that to the out " + \
				    "Attribute."
		self._defaultOutName = "sorted"
		AbstractFunction.__init__ (self, parent, bridge)


class IndexSort (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "index_sort"
		self._displayName = "Index Sort"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/sortindex.png")
		self._description = "This function takes an Attribute in, sorts it by index, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "sorted"
		AbstractFunction.__init__ (self, parent, bridge)


class AbsoluteValue (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "abs"
		self._displayName = "Absolute Value"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/absval.png")
		self._description = "This function takes an Attribute in, takes its absolute value, then sets " + \
				    "that to the out Attribute."
		self._defaultOutName = "abs_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Log (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "log"
		self._displayName = "Natural Log"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/log.png")
		self._description = "This function takes an Attribute in, takes its natural log, then sets " + \
				    "that to the out Attribute."
		self._defaultOutName = "log_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Log10 (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "log10"
		self._displayName = "Log 10"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/log10.png")
		self._description = "This function takes an Attribute in, takes its log base 10, then sets " + \
				    "that to the out Attribute."
		self._defaultOutName = "log_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Exp (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "exp"
		self._displayName = "Natural Exponential"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/exp.png")
		self._description = "This function takes an Attribute in, takes it as the exponent of e, then " + \
				    "sets that to the out Attribute."
		self._defaultOutName = "exp_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Exp10 (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "exp10"
		self._displayName = "Exponential 10"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/exp10.png")
		self._description = "This function takes an Attribute in, takes it as the exponent of 10, then " + \
				    "sets that to the out Attribute."
		self._defaultOutName = "exp_out"
		AbstractFunction.__init__ (self, parent, bridge)


class SquareRoot (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "sqrt"
		self._displayName = "Square Root"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/squareroot.png")
		self._description = "This function takes an Attribute in, takes its square root, then sets that " + \
				    "to the out Attribute."
		self._defaultOutName = "sqrt_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Sin (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "sin"
		self._displayName = "Sin"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/sin.png")
		self._description = "This function takes an Attribute in, takes its sin, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "sin_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Cos (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "cos"
		self._displayName = "Cos"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/cos.png")
		self._description = "This function takes an Attribute in, takes its cos, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "cos_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Cosh (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "cosh"
		self._displayName = "Cosh"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/cosh.png")
		self._description = "This function takes an Attribute in, takes its cosh, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "cosh_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Sinh (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "sinh"
		self._displayName = "Sinh"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/sinh.png")
		self._description = "This function takes an Attribute in, takes its sinh, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "sinh_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Acos (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "acos"
		self._displayName = "Acos"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/arccos.png")
		self._description = "This function takes an Attribute in, takes its acos, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "acos_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Asin (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "asin"
		self._displayName = "Asin"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/arcsin.png")
		self._description = "This function takes an Attribute in, takes its asin, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "asin_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Acosh (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "acosh"
		self._displayName = "Acosh"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/acosh.png")
		self._description = "This function takes an Attribute in, takes its acosh, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "acosh_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Asinh (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "asinh"
		self._displayName = "Asinh"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/asinh.png")
		self._description = "This function takes an Attribute in, takes its asinh, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "asinh_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Atan (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "atan"
		self._displayName = "Atan"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/atan.png")
		self._description = "This function takes an Attribute in, takes its atan, then sets that to " + \
				    "the out Attribute."
		self._defaultOutName = "atan_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Pow (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "pow"
		self._displayName = "Pow"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/pow.png")
		self._description = "This function takes an Attribute in, takes it to the nth power, then sets " + \
				    "that to the out Attribute."
		self._defaultOutName = "pow_out"
		AbstractFunction.__init__ (self, parent, bridge)

		self._exp = PyQt4.QtGui.QSpinBox ()
		self._exp.setRange (0, 100)
		self._exp.setSingleStep (1)
		self.rightLayout.addWidget (PyQt4.QtGui.QLabel ("Exponent:"), 1, 0, 1, 2, PyQt4.QtCore.Qt.AlignRight)
		self.rightLayout.addWidget (self._exp, 1, 2, 1, 2, PyQt4.QtCore.Qt.AlignLeft)

	def Execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		commands = []
		commands.append (self._out.text () + " = " + self._functionName + " (" +
			self._in._shellAccess + ", " + str (self._exp.value ()) + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()


class Real (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "real"
		self._displayName = "Real"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/real.png")
		self._description = "This function takes an Attribute in, takes its real part, then sets that " + \
				    "to the out Attribute."
		self._defaultOutName = "real_out"
		AbstractFunction.__init__ (self, parent, bridge)


class Imaginary (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._functionName = "imag"
		self._displayName = "Imaginary"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/imaginary.png")
		self._description = "This function takes an Attribute in, takes it's imaginary part, then sets " + \
				    "that to the out Attribute."
		self._defaultOutName = "imag_out"
		AbstractFunction.__init__ (self, parent, bridge)


class AttributeHolder (PyQt4.QtGui.QLineEdit):
	def __init__ (self, bridge, parent):
		PyQt4.QtGui.QLineEdit.__init__ (self, "None")
		self.setReadOnly (True)
		self._shellAccess = None
		self._name = None
		self._bridge = bridge
		self._parent = parent

	def clear (self):
		self.setText ("None")
		self._shellAccess = None
		self._name = None

	def dropEvent (self, ev):
		# Get the drug item and then make sure something is indeed there
		data = self._bridge.PopDrugItem ()
		if not data:
			return
		# If its already an attribute, go ahead and drop it
		if data.Type () == "ATTRIBUTE" or (data.Type () == "VALUE" and data._type == "_basin.Attribute"):
			self.setText (data._name)
			self._name = data._name
			self._shellAccess = data._shellAccess
			ev.setDropAction (PyQt4.QtCore.Qt.LinkAction)
			ev.accept ()
		# otherwise just ignore the action
		else:
			ev.ignore ()

     	def dragEnterEvent (self, ev):
		ev.setDropAction (PyQt4.QtCore.Qt.LinkAction)
		ev.accept ()

class NewAttributeHolder (PyQt4.QtGui.QLineEdit):
	def __init__ (self, default):
		PyQt4.QtGui.QLineEdit.__init__ (self, default)
		self._default = default

	def clear (self):
		self.setText (self._default)

	def dropEvent (self, ev):
		ev.ignore ()

	def dragEnterEvent (self, ev):
		ev.ignore ()


class AttributeConstant (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._displayName = "Constant Attribute"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/constant.png")
		self._description = "Choose the number of elements of your new attribute.  Also " + \
			"choose a seed.  Note that if always use the same seed, you always get the " + \
			"same attribute back."
		AbstractFunction.__init__ (self, parent, bridge)

	def _addDataSelection (self):
		self._num = PyQt4.QtGui.QSpinBox ()
		self._num.setRange (1, 999999999)
		self._num.setSingleStep (1)
		self._value = PyQt4.QtGui.QDoubleSpinBox ()
		self._value.setRange (-999999999.9, 999999999.9)
		self._value.setValue (0.0)
		self._value.setSingleStep (1)
		self._out = NewAttributeHolder ("const_out")

		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		rightLayout = PyQt4.QtGui.QGridLayout ()
		rightLayout.addWidget (PyQt4.QtGui.QLabel ("Size:"), 0, 0)
		rightLayout.addWidget (self._num, 0, 1)
		rightLayout.addWidget (PyQt4.QtGui.QLabel ("Value:"), 1, 0)
		rightLayout.addWidget (self._value, 1, 1)
		rightLayout.addWidget (PyQt4.QtGui.QLabel ("Out Attribute:"), 2, 0)
		rightLayout.addWidget (self._out, 2, 1)
		outline.setLayout (rightLayout)

		return outline

	def clear (self):
		self._num.setValue (1)
		self._value.setValue (1)
		self._out.clear ()

	def Execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		self._bridge._ipyShell.Command (self._out.text () + " = Attribute (" +
			str (self._num.value ()) + ", " + str (self._value.value ()) + ")")

		self.clear ()

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		if attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and attr._type == "_basin.Attribute"):
			self._in.setText (attr._name)
			self._in._name = attr._name
			self._in._shellAccess = attr._shellAccess


class AttributeIndex (Widget.DockWidget):
	def __init__ (self, parent, bridge):
		self._type = "General Function"
		self._displayName = "Attribute Index"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = "This function will return an attribute the same size as your " + \
			"input.  It the attribute it returns will provide indices into your input " + \
			"data."
		Widget.DockWidget.__init__ (self, parent, bridge)

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout ()
		box.setLayout (layout)

		executeButton = Constants.BQPushButton ("Execute", layout)
		PyQt4.QtCore.QObject.connect (executeButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.Execute)

		clearButton = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (clearButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def _addDataSelection (self):
		box = PyQt4.QtGui.QGroupBox ("Data Selection")
		layout = PyQt4.QtGui.QVBoxLayout ()
		box.setLayout (layout)

		outLayout = PyQt4.QtGui.QHBoxLayout ()
		outLabel = PyQt4.QtGui.QLabel ("out:")
		outLabel.setAlignment (PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignRight)
		self._out = PyQt4.QtGui.QLineEdit ("index_out")
		outLayout.addWidget (outLabel)
		outLayout.addWidget (self._out)
		layout.addLayout (outLayout)

		self._data = Holders.GenericInputHolder (self._bridge, layout, "data", \
			["ATTRIBUTE"], ["_basin.Attribute", "list"])
		PyQt4.QtCore.QObject.connect (self._data, PyQt4.QtCore.SIGNAL ("textChanged (const QString&)"), self.checkOrder)
		PyQt4.QtCore.QObject.connect (self._data, PyQt4.QtCore.SIGNAL ("textChanged (const QString&)"), self.checkDims)

		dirLayout = PyQt4.QtGui.QHBoxLayout ()
		dirLabel = PyQt4.QtGui.QLabel ("dir:")
		dirLabel.setAlignment (PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignRight)
		self._dir = PyQt4.QtGui.QComboBox ()
		dirLayout.addWidget (dirLabel)
		dirLayout.addWidget (self._dir)

		orderLayout = PyQt4.QtGui.QHBoxLayout ()
		orderLabel = PyQt4.QtGui.QLabel ("order:")
		orderLabel.setAlignment (PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignRight)
		self._order = PyQt4.QtGui.QComboBox ()
		self._order.setEnabled (False)
		self._order.addItem ("Normal", PyQt4.QtCore.QVariant ("0"))
		self._order.addItem ("FFT Half Hermitian", PyQt4.QtCore.QVariant ("1"))
		orderLayout.addWidget (orderLabel)
		orderLayout.addWidget (self._order)

		layout.addLayout (dirLayout)
		layout.addLayout (orderLayout)
		return box

	def checkOrder (self, text):
		# Make sure there's an item in there...
		text = str (text)
		if text != "None":
			type = self._data.type ()
			if type == "<type 'list'>":
				self._order.setEnabled (True)
				return
		self._order.setEnabled (False)

	def checkDims (self, text):
		# Make sure there's an item in there...
		self._dir.clear ()
		text = str (text)
		if text != "None":
			size = self._data.size ()
			for i in xrange (size):
				self._dir.addItem (str (i))


	def clear (self):
		self._data.clear ()
		self._out.setText ("index_out")

	def Execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		order = ""
		if self._order.isEnabled ():
			order += ", " + str (self._order.currentText ())

		self._bridge._ipyShell.Command (self._out.text () + " = attribute_index (" + \
			self._data._item._shellAccess + ", " + str (self._dir.currentText ()) + \
			order + ")", True)
		self.clear ()

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		if attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and attr._type == "_basin.Attribute") \
		or (attr.Type () == "VALUE" and attr._type == "list"):
			self._data.SetItem (attr, False)


class AttributeRandom (AbstractFunction):
	def __init__ (self, parent, bridge):
		self._displayName = "Random Attribute"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/general_functions/random.png")
		self._description = "Choose the number of elements of your new attribute.  Also " + \
			"choose a seed.  Note that if always use the same seed, you " + \
			"always get the same attribute back."
		AbstractFunction.__init__ (self, parent, bridge)

	def _addDataSelection (self):
		self._num = PyQt4.QtGui.QSpinBox ()
		self._num.setRange (1, 999999999)
		self._num.setSingleStep (1)
		self._seed = PyQt4.QtGui.QSpinBox ()
		self._seed.setRange (1, 999999999)
		self._seed.setSingleStep (1)
		self._out = NewAttributeHolder ("rand_out")

		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		rightLayout = PyQt4.QtGui.QGridLayout ()
		rightLayout.addWidget (PyQt4.QtGui.QLabel ("Size:"), 0, 0)
		rightLayout.addWidget (self._num, 0, 1)
		rightLayout.addWidget (PyQt4.QtGui.QLabel ("Seed:"), 1, 0)
		rightLayout.addWidget (self._seed, 1, 1)
		rightLayout.addWidget (PyQt4.QtGui.QLabel ("Out Attribute:"), 2, 0)
		rightLayout.addWidget (self._out, 2, 1)
		outline.setLayout (rightLayout)

		return outline

	def clear (self):
		self._num.setValue (1)
		self._seed.setValue (1)
		self._out.clear ()

	def Execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		commands = []
		commands.append (self._out.text () + " = attribute_random" +
			" (" + str (self._num.value ()) + ", " + str (self._seed.value ()) + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		if attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and attr._type == "_basin.Attribute"):
			self._in.setText (attr._name)
			self._in._name = attr._name
			self._in._shellAccess = attr._shellAccess
