import PyQt4.QtCore
import PyQt4.QtGui

import Constants
import Widget
import Holders

class AbstractLinearAlgebra (Widget.DockWidget):
	def __init__ (self, parent, bridge):
		self._type = "Linear Algebra"
		Widget.DockWidget.__init__ (self, parent, bridge)

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout (box)

		executeButton = Constants.BQPushButton ("Execute", layout)
		executeButton.setDefault (True)
		PyQt4.QtCore.QObject.connect (executeButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.execute)

		clearButton = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (clearButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def _addDataSelection (self):
		self._inHolders = []
		self._outHolder = ""

		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		dataLayout = PyQt4.QtGui.QVBoxLayout (outline)
		for input in self._inList:
			holder = Holders.GenericInputHolder (self._bridge, dataLayout, input, [], ["_basin.matrix"])

		dataLayout.addSpacing (10)

		output = self._outList
		tempLayout = Constants.BQHBoxLayout (dataLayout)
		label = PyQt4.QtGui.QLabel (output + ":")
		label.setAlignment (PyQt4.QtCore.Qt.AlignRight | PyQt4.QtCore.Qt.AlignVCenter)
		tempLayout.addWidget (label)
		holder = Holders.OutputHolder (output, self._bridge)
		self._outHolder = holder
		tempLayout.addWidget (holder)

		return outline

	def clear (self):
		for holder in self._inHolders:
			holder.clear ()
		self._outHolder.clear ()

	def DoubleClicked (self, matrix = None):
		if matrix == None:
			return

		if matrix.Type () == "VALUE" and matrix._type == "_basin.matrix":
			for holder in self._inHolders:
				if holder._item == None:
					holder.SetItem (matrix, False)
					return True
		return False

	def execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False
		if self._outHolder == "":
			return False

		inString = ""
		for holder in self._inHolders:
			inString += holder._item._shellAccess + ", "
		inString = inString[:-2]

		commands = []
		commands.append (self._outHolder.text () + " = " + self._functionName + " (" + inString + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()


class  MatrixInvert (AbstractLinearAlgebra):
	def __init__ (self, parent, bridge):
		self._displayIcon =  PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/linear_algebra/matrixinvert.png")
		self._displayName = "Invert Matrix"
		self._functionName = "matrix_invert"
		self._description = "This function will invert a square matrix."
		self._inList = ["matrix"]
		self._outList = "inverse"
		AbstractLinearAlgebra.__init__ (self, parent, bridge)


class  MatrixMult (AbstractLinearAlgebra):
	def __init__ (self, parent, bridge):
		self._displayIcon =  PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/linear_algebra/matrixmult.png")
		self._displayName = "Multiply Matrices"
		self._functionName = "matrix_mult"
		self._description = "This function will multiply two different matrices."
		self._inList = ["a", "b"]
		self._outList = "product"
		AbstractLinearAlgebra.__init__ (self, parent, bridge)


class MatrixCreate (AbstractLinearAlgebra):
	def __init__ (self, parent, bridge):
		self._type = "Linear Algebra"
		self._displayName = "Create Matrix"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/linear_algebra/matrixcreate.png")
		self._description = "Supply a python list and the dimensions to create a matrix."
		AbstractLinearAlgebra.__init__ (self, parent, bridge)

	def _addDataSelection (self):
		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		dataLayout = PyQt4.QtGui.QVBoxLayout (outline)

		self._dataHolder = Holders.GenericInputHolder (self._bridge, dataLayout, "data", [], ["_basin.matrix"])

		dimMLayout = Constants.BQHBoxLayout (dataLayout)
		dimMLabel = PyQt4.QtGui.QLabel ("m:")
		dimMLabel.setAlignment (PyQt4.QtCore.Qt.AlignRight | PyQt4.QtCore.Qt.AlignVCenter)
		self._dimM = PyQt4.QtGui.QSpinBox ()
		self._dimM.setRange (1,999999999)
		dimMLayout.addWidget (dimMLabel)
		dimMLayout.addWidget (self._dimM)

		dimNLayout = Constants.BQHBoxLayout (dataLayout)
		dimNLabel = PyQt4.QtGui.QLabel ("n:")
		dimNLabel.setAlignment (PyQt4.QtCore.Qt.AlignRight | PyQt4.QtCore.Qt.AlignVCenter)
		self._dimN = PyQt4.QtGui.QSpinBox ()
		self._dimN.setRange (1,999999999)
		dimNLayout.addWidget (dimNLabel)
		dimNLayout.addWidget (self._dimN)

		dataLayout.addSpacing (10)
		output = "out"
		tempLayout = Constants.BQHBoxLayout (dataLayout)
		label = PyQt4.QtGui.QLabel (output + ":")
		label.setAlignment (PyQt4.QtCore.Qt.AlignRight | PyQt4.QtCore.Qt.AlignVCenter)
		tempLayout.addWidget (label)
		holder = Holders.OutputHolder (output, self._bridge)
		self._outHolder = holder
		tempLayout.addWidget (holder)

		return outline

	def clear (self):
		self._dimM.setValue (1)
		self._dimN.setValue (1)
		self._dataHolder.clear ()
		self._outHolder.clear ()

	def DoubleClicked (self, list = None):
		if list == None:
			return

		if list.Type () == "VALUE" and list._type == "list":
			holder = self._dataHolder
			if holder._item == None:
				holder.SetItem (list, False)
				return True
		return False

	def execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False
		if self._outHolder == "":
			return False

		m, n = self._dimM.value (), self._dimN.value ()
		dataLength = self._dataHolder.getLength ()
		if (m * n) != dataLength and self._dataHolder._item != None:
			self._bridge._statusBar.showTimed ("Dimensions provided don't fit data! (" + \
				str (dataLength) + " != " + str (m) + " * " + str (n) + ")", STATUS_WAIT)
			return False

		data = ""
		if self._dataHolder._item != None:
			data = self._dataHolder._item._shellAccess + ", "

		commands = []
		commands.append (self._outHolder.text () + " = matrix (" + data + str (m) + ", " + str (n) + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()


class IdentityMatrixCreate (AbstractLinearAlgebra):
	def __init__ (self, parent, bridge):
		self._type = "Linear Algebra"
		self._displayName = "Create Identity Matrix"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/linear_algebra/identitymatrixcreate.png")
		self._description = "Supply the square dimension of the identity matrix."
		AbstractLinearAlgebra.__init__ (self, parent, bridge)

	def _addDataSelection (self):
		outline = PyQt4.QtGui.QGroupBox ("Data Selection")
		dataLayout = PyQt4.QtGui.QVBoxLayout (outline)

		dimLayout = Constants.BQHBoxLayout (dataLayout)
		dimLabel = PyQt4.QtGui.QLabel ("dim:")
		dimLabel.setAlignment (PyQt4.QtCore.Qt.AlignRight | PyQt4.QtCore.Qt.AlignVCenter)
		self._dim = PyQt4.QtGui.QSpinBox ()
		self._dim.setRange (1,999999999)
		dimLayout.addWidget (dimLabel)
		dimLayout.addWidget (self._dim)

		dataLayout.addSpacing (10)
		output = "out"
		tempLayout = Constants.BQHBoxLayout (dataLayout)
		label = PyQt4.QtGui.QLabel (output + ":")
		label.setAlignment (PyQt4.QtCore.Qt.AlignRight | PyQt4.QtCore.Qt.AlignVCenter)
		tempLayout.addWidget (label)
		holder = Holders.OutputHolder (output, self._bridge)
		self._outHolder = holder
		tempLayout.addWidget (holder)

		return outline

	def clear (self):
		self._dim.setValue (1)
		self._outHolder.clear ()

	def DoubleClicked (self, list = None):
		return

	def execute (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False
		if self._outHolder == "":
			return False

		commands = []
		commands.append (self._outHolder.text () + " = matrix (" + str (self._dim.value ()) + ")")
		self._bridge._ipyShell.BatchCommands (commands, True)

		self.clear ()
