import PyQt4.QtCore
import PyQt4.QtGui

import time

import Constants
import DataGuesser
import Widget
import Holders

class Gnuplot (Widget.DockWidget):
	def __init__ (self, layout, bridge):
		self._type = "Plotting"
		self._displayName = "Gnuplot"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/plotting/plot.png")
		self._description = "Simply fill one, or both, of the attribute holders.  Clicking " + \
			"`Plot Data` will create a Gnuplot window on the cluster."
		Widget.DockWidget.__init__ (self, layout, bridge)

	def _addDataSelection (self):
		box = PyQt4.QtGui.QGroupBox ("Data Selection")
		layout = PyQt4.QtGui.QVBoxLayout (box)

		self._plotName = Constants.BTextField (layout, "Plot Title", "My Plot")
		self._group =  Holders.FreeHolderGroup (self._bridge, ["x", "y"])
		layout.addLayout (self._group.layout)
		return box

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout (box)

		self.plotButton = Constants.BQPushButton ("Plot Data", layout)
		self.plotButton.setDefault (True)
		PyQt4.QtCore.QObject.connect (self.plotButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.plot)

		self.clearButton = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (self.clearButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def clear (self):
		for holder in self._group._inList:
			holder.clear ()
		self._plotName.setText ("My Plot")

	def guessDefaults (self):
		guessList = []
		guessList.append (DataGuesser.GenerateRegExpFromName ("X"))
		guessList.append (DataGuesser.GenerateRegExpFromName ("Y"))
		DataGuesser.GuessAttributes (self, guessList)

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		if attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and \
			attr._type == "_basin.Attribute"):
			for holder in self._group._inList:
				if holder.SetItem (attr, clobber = False) == True:
					return

	def plot (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			self._bridge._statusBar.showTimed ("You need to be connected and in parallel mode!", STATUS_WAIT)
			return False

		accesses = self._group.getInAccesses (False)
		if len (accesses) == 0:
			self._bridge._statusBar.showTimed ("You must have at least one argument!", STATUS_WAIT)
			return

		attributes = ""
		for attr in accesses:
			attributes += attr + ", "
		self._bridge._ipyShell.Command ("plot (" + attributes + "\"" + \
			str (self._plotName.displayText ()) + "\")")
		self.clear ()


class VisItPlot (Widget.DockWidget):
	def __init__ (self, layout, bridge):
		self._visitModuleLoaded = bridge._importVisIt

		self._type = "Plotting"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DIR + "/plotting/visit.png")
		self._displayName = "VisIt"

		if self._visitModuleLoaded:
			self._description = "VisIt's python module has been loaded.  Simply plot your data and the client will take care of connecting VisIt to the cluster and drawing the plot."
		else:
			self._description = "VisIt's python module was not found.  You may expose your data here, but you will need to open VisIt connect to your cluster manually, and plot the data yourself."

		self._visitStarted = False
		Widget.DockWidget.__init__ (self, layout, bridge)
		self.continuousButton.click ()

		self._exposesDiscrete = [[], []]
		self._exposesContinuous = [[], []]

		if self._visitModuleLoaded:
			self._visit = VisItThread (bridge, self)
			self._visitWindows = 0
			self._visit.start ()

	def Update (self, discrete, discreteWindows, continuous, continuousWindows):
		self._exposesDiscrete[0] = discrete
		self._exposesDiscrete[1] = discreteWindows
		self._exposesContinuous[0] = continuous
		self._exposesContinuous[1] = continuousWindows

	def _addHeader (self):
		layout = PyQt4.QtGui.QVBoxLayout ()
		headerLayout = Widget.DockWidget._addHeader (self)
		layout.addLayout (headerLayout)

		self._plotName = Constants.BTextField (layout, "Plot Title", "My Plot")

		return layout

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout (box)

		if self._visitModuleLoaded:
			self._commandExpose = Constants.BQPushButton ("Plot", layout)
		else:
			self._commandExpose = Constants.BQPushButton ("Expose", layout)
		PyQt4.QtCore.QObject.connect (self._commandExpose, PyQt4.QtCore.SIGNAL ("clicked ()"), self.Expose)

		if self._visitModuleLoaded:
			#self._commandClose = Constants.BQPushButton ("Close VisIt", layout, False)
			#PyQt4.QtCore.QObject.connect (self._commandClose, PyQt4.QtCore.SIGNAL ("clicked ()"), self.DeleteVisItThread)

			self._commandManage = Constants.BQPushButton ("Manage Plots", layout, False)
			PyQt4.QtCore.QObject.connect (self._commandManage, PyQt4.QtCore.SIGNAL ("clicked ()"), self.ManagePlots)
		else:
			commandLaunch = Constants.BQPushButton ("Launch", layout)
			PyQt4.QtCore.QObject.connect (commandLaunch, PyQt4.QtCore.SIGNAL ("clicked ()"), self.LaunchVisIt)

		commandClear = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (commandClear, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def _addDataSelection (self):
		self._continuousMode = self.createContinuousMode ()
		self._discreteMode = self.createDiscreteMode ()

		box = PyQt4.QtGui.QGroupBox ("Data Selection")
		layout = PyQt4.QtGui.QHBoxLayout (box)
		layout.addWidget (self._continuousMode)
		layout.addWidget (self._discreteMode)
		return box

	def _addModes (self):
		box = PyQt4.QtGui.QGroupBox ("Modes")
		layout = PyQt4.QtGui.QHBoxLayout (box)

		self.continuousButton = PyQt4.QtGui.QRadioButton ("Continuous", self)
		PyQt4.QtCore.QObject.connect (self.continuousButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.setContinuousMode)
		layout.addWidget (self.continuousButton)

		discreteButton = PyQt4.QtGui.QRadioButton ("Discrete", self)
		PyQt4.QtCore.QObject.connect (discreteButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.setDiscreteMode)
		layout.addWidget (discreteButton)

		return box

	def _addOptions (self):
		# No options needed if visit's python module isn't found
		if self._visitModuleLoaded:
			box = PyQt4.QtGui.QGroupBox ("Initial Options")
			layout = PyQt4.QtGui.QHBoxLayout (box)

			self.stereoCheck = PyQt4.QtGui.QCheckBox ("Stereo")
			layout.addWidget (self.stereoCheck)

			self.invertBgCheck = PyQt4.QtGui.QCheckBox ("Inverted BG")
			layout.addWidget (self.invertBgCheck)

			return box

	def guessDefaults (self):
		guessList = []
		if   self._continuousMode.isVisible () == True:
			pass
		elif self._discreteMode.isVisible () == True:
			guessList.append (DataGuesser.GenerateRegExpFromName ("X"))
			guessList.append (DataGuesser.GenerateRegExpFromName ("Y"))
			guessList.append (DataGuesser.GenerateRegExpFromName ("Z"))
			DataGuesser.GuessAttributes (self, guessList)

	def LaunchVisIt (self):
		if self._bridge._connected and self._bridge._parallel:
			self._bridge.makeLocal ()
			self._bridge._ipyShell.SilentCommand ("subprocess.Popen([\"visit\"])")
			self._bridge.makeParallel ()

	def StartVisItThread (self):
		# If this is the first expose, launch it (and check for stereo checkbox)
		if not self._visit.launched ():
			# launches it with the option of stereo viewing
			stereo = False
			if self.stereoCheck.checkState () == PyQt4.QtCore.Qt.Checked:
				stereo = True
			self.emit (PyQt4.QtCore.SIGNAL ("threadLaunch (PyQt_PyObject)"), stereo)
			self._visitWindows = 1

			if self.invertBgCheck.checkState () == PyQt4.QtCore.Qt.Checked:
				self.emit (PyQt4.QtCore.SIGNAL ("threadInvertBackgroundColor ()"))

			# meaningless to change this after launch, so don't let the user do it
			self.stereoCheck.setEnabled (False)
			self.invertBgCheck.setEnabled (False)

			self._commandManage.setEnabled (True)

			# connect to the host
			self.emit (PyQt4.QtCore.SIGNAL ("threadOpenComputeEngine ()"))

			self.emit (PyQt4.QtCore.SIGNAL ("threadOpenSimulation ()"))
			return True

		return False

	def DeleteVisItThread (self):
		# No need to detroy the object if it hasn't been launched yet
		if not self._visit.launched ():
			self._bridge._statusBar.showTimed ("VisIt has not been started!", Constants.STATUS_WAIT)
			return

		self.emit (PyQt4.QtCore.SIGNAL ("threadClose ()"))
		del (self._visit)
		self._visit = VisItThread (self._bridge, self)
		self._visitWindows = 0
		self._visit.start ()

		self.stereoCheck.setEnabled (True)
		self.invertBgCheck.setEnabled (True)

		self._commandClose.setEnabled (False)
		self._commandManage.setEnabled (False)

	def ManagePlots (self):
		if not self._visit.launched ():
			self._bridge._statusBar.showTimed ("VisIt has not been started!", Constants.STATUS_WAIT)
			return

		dialog = PyQt4.QtGui.QDialog ()
		dialog.setWindowIcon (self._displayIcon)
		dialog.setWindowTitle ("VisIt Plot Manager")
		mainLayout = PyQt4.QtGui.QVBoxLayout (dialog)

		header = PyQt4.QtGui.QLabel ("<big><b>VisIt Plot Manager</b></big>")
		header.setTextFormat (PyQt4.QtCore.Qt.RichText)
		mainLayout.addWidget (header)

		if len (self._exposesDiscrete[0]) > 0:
			header = PyQt4.QtGui.QLabel ("<b>Discrete Plots</b>")
			header.setTextFormat (PyQt4.QtCore.Qt.RichText)
			mainLayout.addWidget (header)

			for p in xrange (len (self._exposesDiscrete[0])):
				layout = Constants.BQHBoxLayout (mainLayout)
				label = PyQt4.QtGui.QLabel (self._exposesDiscrete[0][p])
				layout.addWidget (label)
				combo = PyQt4.QtGui.QComboBox ()
				for i in xrange (16):
					combo.addItem (str (i + 1))
				combo.setCurrentIndex (int (self._exposesDiscrete[1][p]))
				layout.addWidget (combo)
		
		if len (self._exposesContinuous[0]) > 0:
			header = PyQt4.QtGui.QLabel ("<b>Continuous Plots</b>")
			header.setTextFormat (PyQt4.QtCore.Qt.RichText)
			mainLayout.addWidget (header)

			for p in xrange (len (self._exposesContinuous[0])):
				layout = Constants.BQHBoxLayout (mainLayout)
				label = PyQt4.QtGui.QLabel (self._exposesContinuous[0][p])
				layout.addWidget (label)
				combo = PyQt4.QtGui.QComboBox ()
				for i in xrange (16):
					combo.addItem (str (i + 1))
				combo.setCurrentIndex (int (self._exposesContinuous[1][p]))
				layout.addWidget (combo)

		buttonLayout = Constants.BQHBoxLayout (mainLayout)
		apply = Constants.BQPushButton ("Apply", buttonLayout)
		close = Constants.BQPushButton ("Close", buttonLayout)
		PyQt4.QtCore.QObject.connect (close, PyQt4.QtCore.SIGNAL ("clicked ()"), dialog.accept)

		dialog.exec_ ()

	def StartVisIt (self):
		if self._visitStarted == False:
			self._visitStarted = True
			result = self._bridge._ipyShell.Command ("visit_module_start ()")
			if result.find ("rror") != -1:
				self._bridge.Error ("BASIN_plotting.py", "VisitPlot", "StartVisIt")

	def CheckContext (self):
		attribute = self._continuous._item._shellAccess
		result =  self._bridge._ipyShell.SilentCommand ("print \"\\n\" + str (type (" + attribute + \
			".get_context ()))")
		result = result.split ("\n")[3]
		if result == "<class '_basin.Context'>":
			return True
		self._bridge._statusBar.showTimed ("This Attribute does not have a Context!", Constants.STATUS_WAIT)
		return False

	def Expose (self):
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			return False

		self.StartVisIt ()
		result = ""

		# Continuous Mode Exposing
		if   self._continuousMode.isVisible () == True:
			# Can't do this without a working context
			if self.CheckContext ():
				grid = self._continuous._item._shellAccess
				context = grid + ".get_context ()"
				nx = context + ".get_n_grid_points ()"
				ll = context + ".get_LowerCorner ()"
				ur = context + ".get_UpperCorner ()"
				result = self._bridge._ipyShell.Command ("visit_expose_continuous (" + \
					ll + ", " + ur + ", " + nx + ", " + self._continuous._item._shellAccess + \
					", \"" + str (self._plotName.text ()) + "\")")

		# Discrete Mode Exposing
		elif self._discreteMode.isVisible () == True:
			attributes = self._discrete.getInAccesses (False)
			weight = self._discreteWeight.getInAccesses (False)

			weightCreated = False
			# Use the weight supplied by the user
			if len (weight) == 1:
				weight = weight[0]
			# No weight supplied, create a temporary one
			else:
				weightCreated = True
				weight = "BASIN_TEMP_WEIGHT"
				size = self._bridge._ipyShell.SilentCommand ("print \"\\n\" + str (" + attributes[0] + \
					".size ())").split ("\n")[3]
				self._bridge._ipyShell.Command ("BASIN_TEMP_WEIGHT = Attribute (" + size + ", 0.0)")

			if   len (attributes) == 3:
				result = self._bridge._ipyShell.Command ("visit_expose_discrete (" + \
					attributes[0] + ", " + attributes[1] + ", " + attributes[2] + \
					", " + weight + ", \"" + str (self._plotName.text ()) + "\")")
			if   len (attributes) == 2:
				result = self._bridge._ipyShell.Command ("visit_expose_discrete (" + \
					attributes[0] + ", " + attributes[1] + \
					", " + weight + ", \"" + str (self._plotName.text ()) + "\")")
			# If you weren't given a weight, then you created one on the fly, so delete it
			if weightCreated:
				self._bridge._ipyShell.Command ("del (BASIN_TEMP_WEIGHT)")

		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_plotting.py", "VisItPlot", "Expose", result)
			return

	def clearMode (self):
		self._continuousMode.setVisible (False)
		self._discreteMode.setVisible (False)

	def setContinuousMode (self):
		self.clearMode ()
		self._continuousMode.setVisible (True)
		self.guessDefaults ()

	def setDiscreteMode (self):
		self.clearMode ()
		self._discreteMode.setVisible (True)
		self.guessDefaults ()

	def createContinuousMode (self):
		widget = PyQt4.QtGui.QWidget ()
		widget.setVisible (False)
		layout = PyQt4.QtGui.QVBoxLayout (widget)

		self._continuous = Holders.GenericInputHolder (self._bridge, layout, "Grid Attribute", \
			["ATTRIBUTE"], ["_basin.Attribute"])
		PyQt4.QtCore.QObject.connect (self._continuous, PyQt4.QtCore.SIGNAL ("textChanged (const QString&)"), self.setPlotName)

		return widget

	def createDiscreteMode (self):
		widget = PyQt4.QtGui.QWidget ()
		widget.setVisible (False)
		layout = PyQt4.QtGui.QVBoxLayout (widget)

		self._discrete =  Holders.FreeHolderGroup (self._bridge, ["x", "y", "z"])
		self._discreteWeight = Holders.FreeHolderGroup (self._bridge, ["weight"])
		PyQt4.QtCore.QObject.connect (self._discreteWeight._inList[0], PyQt4.QtCore.SIGNAL ("textChanged (const QString&)"), self.setPlotName)

		layout.addLayout (self._discrete.layout)
		layout.addLayout (self._discreteWeight.layout)

		return widget

	def clear (self):
		if   self._continuousMode.isVisible () == True:
			self._continuous.clear ()
		elif self._discreteMode.isVisible () == True:
			self._discrete.clear ()
			for holder in self._discrete._inList:
				holder.clear ()
			for holder in self._discreteWeight._inList:
				holder.clear ()
		self._plotName.setText ("My Plot")

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		if   self._continuousMode.isVisible () == True:
			if attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and \
			attr._type == "_basin.Attribute"):
				self._continuous.SetItem (attr, clobber = False)
		elif self._discreteMode.isVisible () == True:
			if attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and \
			attr._type == "_basin.Attribute"):
				for holder in self._discrete._inList:
					if holder.SetItem (attr, clobber = False) == True:
						return
				for holder in self._discreteWeight._inList:
					if holder.SetItem (attr, clobber = False) == True:
						return

	def setPlotName (self, name):
		self._plotName.setText (name)


class PyLab (Widget.DockWidget):
	def __init__ (self, layout, bridge):
		self._type = "Plotting"
		self._displayName = "PyLab"
		self._displayIcon = PyQt4.QtGui.QIcon (Constants.ICON_DEFAULT)
		self._description = "To plot using pylab you simply have to fill in two pieces of data: " + \
			"The x and y components.  They may either be a python list, or basin attribute, or " + \
			"any combination of the two.  Please Note: by doing this plot you will completely " + \
			"stop any interaction with the client's interface until the plot is then closed."
		Widget.DockWidget.__init__ (self, layout, bridge)
		self.scatterButton.click ()

	def _addCommands (self):
		box = PyQt4.QtGui.QGroupBox ("Commands")
		layout = PyQt4.QtGui.QHBoxLayout (box)

		self.plotButton = Constants.BQPushButton ("Plot Data", layout)
		self.plotButton.setDefault (True)
		PyQt4.QtCore.QObject.connect (self.plotButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.plot)

		self.clearButton = Constants.BQPushButton ("Clear", layout)
		PyQt4.QtCore.QObject.connect (self.clearButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.clear)

		return box

	def _addModes (self):
		box = PyQt4.QtGui.QGroupBox ("Modes")
		layout = PyQt4.QtGui.QHBoxLayout (box)

		self.scatterButton = PyQt4.QtGui.QRadioButton ("Scatter", self)
		PyQt4.QtCore.QObject.connect (self.scatterButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.setScatterMode)
		layout.addWidget (self.scatterButton)

		contourButton = PyQt4.QtGui.QRadioButton ("Contour/Color", self)
		PyQt4.QtCore.QObject.connect (contourButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.setContourMode)
		layout.addWidget (contourButton)

		histogramButton = PyQt4.QtGui.QRadioButton ("Histogram", self)
		PyQt4.QtCore.QObject.connect (histogramButton, PyQt4.QtCore.SIGNAL ("clicked ()"), self.setHistogramMode)
		layout.addWidget (histogramButton)

		return box

	def _addOptions (self):
		self._scatterOptions = self.createScatterOptions ()
		self._contourOptions = self.createContourOptions ()
		self._histogramOptions = self.createHistogramOptions ()

		box = PyQt4.QtGui.QGroupBox ("Options")
		layout = PyQt4.QtGui.QHBoxLayout (box)
		layout.addWidget (self._scatterOptions)
		layout.addWidget (self._contourOptions)
		layout.addWidget (self._histogramOptions)
		return box

	def _addDataSelection (self):
		self._scatterPlot = self.createScatterPlot ()
		self._contourPlot = self.createContourPlot ()
		self._histogramPlot = self.createHistogramPlot ()

		box = PyQt4.QtGui.QGroupBox ("Data Selection")
		layout = PyQt4.QtGui.QHBoxLayout (box)
		layout.addWidget (self._scatterPlot)
		layout.addWidget (self._contourPlot)
		layout.addWidget (self._histogramPlot)
		return box

	def clearMode (self):
		self._scatterPlot.setVisible (False)
		self._scatterOptions.setVisible (False)
		self._contourPlot.setVisible (False)
		self._contourOptions.setVisible (False)
		self._histogramPlot.setVisible (False)
		self._histogramOptions.setVisible (False)

	def setScatterMode (self):
		self.clearMode ()
		self._scatterPlot.setVisible (True)
		self._scatterOptions.setVisible (True)
		self.guessDefaults ()

	def setContourMode (self):
		self.clearMode ()
		self._contourPlot.setVisible (True)
		self._contourOptions.setVisible (True)
		self.guessDefaults ()

	def setHistogramMode (self):
		self.clearMode ()
		self._histogramPlot.setVisible (True)
		self._histogramOptions.setVisible (True)
		self.guessDefaults ()

	def createContourOptions (self):
		box = PyQt4.QtGui.QWidget ()
		layout = PyQt4.QtGui.QVBoxLayout (box)

		colorLayout = Constants.BQHBoxLayout (layout)
		colorLabel = PyQt4.QtGui.QLabel ("Plot Colormap:")
		colorLabel.setAlignment (PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignRight)
		colorLayout.addWidget (colorLabel)
		self.contourPlotColor = PyQt4.QtGui.QComboBox ()
		self.contourPlotColor.addItems (["gray", "autumn", "bone", "cool", "copper", "flag", "hot", "hsv", "jet", "pink", "prism", "spring", "summer", "winter", "spectral"])
		colorLayout.addWidget (self.contourPlotColor)

		return box

	def createContourPlot (self):
		box = PyQt4.QtGui.QWidget ()
		layout = PyQt4.QtGui.QVBoxLayout (box)

		namesLayout = Constants.BQHBoxLayout (layout)
		self._contourName = PyQt4.QtGui.QLineEdit ("My Contour Plot")
		namesLayout.addWidget (PyQt4.QtGui.QLabel ("Plot Title:"))
		namesLayout.addWidget (self._contourName)

		dataLayout = Constants.BQVBoxLayout (layout)
		
		self._zContour = Holders.GenericInputHolder (self._bridge, dataLayout, "z", \
			["ATTRIBUTE"], ["_basin.Attribute", "pylab.array"])
		self._contourInputs = [self._zContour]

		self._ContourSwitch = PyQt4.QtGui.QCheckBox("Show Contours")
		self._ContourSwitch.setCheckState(PyQt4.QtCore.Qt.Checked);
		self._ColorSwitch = PyQt4.QtGui.QCheckBox("Show Colorplot")

		dataLayout.addWidget(self._ContourSwitch)
		dataLayout.addWidget(self._ColorSwitch)
		
		return box

	def createScatterOptions (self):
		box = PyQt4.QtGui.QWidget ()
		layout = PyQt4.QtGui.QVBoxLayout (box)

		colorLayout = Constants.BQHBoxLayout (layout)
		colorLabel = PyQt4.QtGui.QLabel ("Plot Color:")
		colorLabel.setAlignment (PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignRight)
		colorLayout.addWidget (colorLabel)
		self.scatterPlotColor = PyQt4.QtGui.QComboBox ()
		self.scatterPlotColor.addItem ("Cyan", PyQt4.QtCore.QVariant ("c"))
		self.scatterPlotColor.addItem ("Magenta", PyQt4.QtCore.QVariant ("m"))
		self.scatterPlotColor.addItem ("Yellow", PyQt4.QtCore.QVariant ("y"))
		self.scatterPlotColor.addItem ("Black", PyQt4.QtCore.QVariant ("k"))
		self.scatterPlotColor.addItem ("Red", PyQt4.QtCore.QVariant ("r"))
		self.scatterPlotColor.addItem ("Green", PyQt4.QtCore.QVariant ("g"))
		self.scatterPlotColor.addItem ("Blue", PyQt4.QtCore.QVariant ("b"))
		colorLayout.addWidget (self.scatterPlotColor)

		styleLayout = Constants.BQHBoxLayout (layout)
		styleLabel = PyQt4.QtGui.QLabel ("Point Style:")
		styleLabel.setAlignment (PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignRight)
		styleLayout.addWidget (styleLabel)
		self.scatterStyle = PyQt4.QtGui.QComboBox ()
		self.scatterStyle.addItem ("Plus", PyQt4.QtCore.QVariant ("+"))
		self.scatterStyle.addItem ("Circle", PyQt4.QtCore.QVariant ("o"))
		self.scatterStyle.addItem ("Dot", PyQt4.QtCore.QVariant ("."))
		self.scatterStyle.addItem ("Square", PyQt4.QtCore.QVariant ("s"))
		self.scatterStyle.addItem ("Cross", PyQt4.QtCore.QVariant ("x"))
		self.scatterStyle.addItem ("Right-Triangle", PyQt4.QtCore.QVariant (">"))
		self.scatterStyle.addItem ("Left-Triangle", PyQt4.QtCore.QVariant ("<"))
		self.scatterStyle.addItem ("Up-Triangle", PyQt4.QtCore.QVariant ("^"))
		self.scatterStyle.addItem ("Down-Triangle", PyQt4.QtCore.QVariant ("v"))
		styleLayout.addWidget (self.scatterStyle)

		return box

	def createHistogramOptions (self):
		box = PyQt4.QtGui.QWidget ()
		layout = PyQt4.QtGui.QVBoxLayout (box)
		return box

	def createHistogramPlot (self):
		box = PyQt4.QtGui.QWidget ()
		layout = PyQt4.QtGui.QVBoxLayout (box)

		namesLayout = Constants.BQHBoxLayout (layout)
		self._histogramName = PyQt4.QtGui.QLineEdit ("My Histogram Plot")
		namesLayout.addWidget (PyQt4.QtGui.QLabel ("Plot Title:"))
		namesLayout.addWidget (self._histogramName)

		dataLayout = Constants.BQVBoxLayout (layout)
		self._dataHistogram = Holders.GenericInputHolder (self._bridge, dataLayout, "hist", \
			["ATTRIBUTE"], ["_basin.Attribute", "list"])
		self._histogramInputs = [self._dataHistogram]

		binsLayout = Constants.BQHBoxLayout (layout)
		binsLabel = PyQt4.QtGui.QLabel ("Bins:")
		binsLabel.setAlignment (PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignRight)
		binsLayout.addWidget (binsLabel)
		self._binsHistogram = PyQt4.QtGui.QSpinBox ()
		self._binsHistogram.setRange (1, 999999999)
		binsLayout.addWidget (self._binsHistogram)
		dataLayout.setAlignment (binsLayout, PyQt4.QtCore.Qt.AlignVCenter | PyQt4.QtCore.Qt.AlignLeft)

		return box

	def createScatterPlot (self):
		box = PyQt4.QtGui.QWidget ()
		layout = PyQt4.QtGui.QVBoxLayout (box)

		namesLayout = Constants.BQHBoxLayout (layout)
		self._scatterName = PyQt4.QtGui.QLineEdit ("My Scatter Plot")
		namesLayout.addWidget (PyQt4.QtGui.QLabel ("Plot Title:"))
		namesLayout.addWidget (self._scatterName)

		dataLayout = Constants.BQVBoxLayout (layout)
		self._xScatter = Holders.GenericInputHolder (self._bridge, dataLayout, "x", \
			["ATTRIBUTE"], ["_basin.Attribute", "list"])
		self._yScatter = Holders.GenericInputHolder (self._bridge, dataLayout, "y", \
			["ATTRIBUTE"], ["_basin.Attribute", "list"])
		self._scatterInputs = [self._xScatter, self._yScatter]

		errorLayout = Constants.BQVBoxLayout (layout)
		xerrorLayout = Constants.BQHBoxLayout (errorLayout)
		xerrorSwitch = PyQt4.QtGui.QCheckBox ("x error:")
		xerrorLayout.addWidget (xerrorSwitch)
		self._xerror = PyQt4.QtGui.QDoubleSpinBox ()
		self._xerror.setEnabled (False)
		self._xerror.setRange (0.0, 999999999.9)
		xerrorLayout.addWidget (self._xerror)
		PyQt4.QtCore.QObject.connect (xerrorSwitch, PyQt4.QtCore.SIGNAL ("stateChanged (int)"), self.toggleXError)

		yerrorLayout = Constants.BQHBoxLayout (errorLayout)
		yerrorSwitch = PyQt4.QtGui.QCheckBox ("y error:")
		yerrorLayout.addWidget (yerrorSwitch)
		self._yerror = PyQt4.QtGui.QDoubleSpinBox ()
		self._yerror.setEnabled (False)
		self._yerror.setRange (0.0, 999999999.9)
		yerrorLayout.addWidget (self._yerror)
		PyQt4.QtCore.QObject.connect (yerrorSwitch, PyQt4.QtCore.SIGNAL ("stateChanged (int)"), self.toggleYError)

		return box

	def toggleXError (self, val):
		if val == 0:
			self._xerror.setEnabled (False)
		else:
			self._xerror.setEnabled (True)

	def toggleYError (self, val):
		if val == 0:
			self._yerror.setEnabled (False)
		else:
			self._yerror.setEnabled (True)

	def DoubleClicked (self, attr = None):
		if attr == None:
			return

		if attr.Type () == "ALIAS":
			attr = attr._item

		inputs = None
		if   self._scatterPlot.isVisible () == True:
			inputs = self._scatterInputs
		elif self._contourPlot.isVisible () == True:
			inputs = self._contourInputs
		elif self._histogramPlot.isVisible () == True:
			inputs = self._histogramInputs

		if   attr.Type () == "ATTRIBUTE" or (attr.Type () == "VALUE" and \
			attr._type == "_basin.Attribute"):
			for holder in inputs:
				if holder.SetItem (attr, clobber = False) == True:
					return
		elif attr.Type () == "VALUE" and attr._type == "list":
			for holder in inputs:
				if holder.SetItem (attr, clobber = False) == True:
					return

	def guessDefaults (self):
		guessList = []

		if   self._scatterPlot.isVisible () == True:
			guessList.append (DataGuesser.GenerateRegExpFromName ("x"))
			guessList.append (DataGuesser.GenerateRegExpFromName ("y"))
		elif self._contourPlot.isVisible () == True:
			guessList.append (DataGuesser.GenerateRegExpFromName ("x"))
			guessList.append (DataGuesser.GenerateRegExpFromName ("y"))
			guessList.append (DataGuesser.GenerateRegExpFromName ("z"))
		elif self._histogramPlot.isVisible () == True:
			guessList.append (DataGuesser.GenerateRegExpFromWord ("hist"))
		DataGuesser.GuessAttributes (self, guessList)

	def plot (self):
		"""
		PyLab.plot (self)

		This method will be called whenever the user clicks on the
		button labeled 'Plot'.  This method will only send a meta-
		command to the ipython command line.  Note that this command
		isn't really a python command, but more of command that the
		fake ipython shell is going to convert into a series of valid
		python commands.
		"""
		if not ( self._bridge._connected == True and self._bridge._parallel == True):
			self._bridge._statusBar.showTimed ("You need to be connected and in parallel mode!", STATUS_WAIT)
			return False
		if not self._bridge._importPylab:
			self._bridge._statusBar.showTimed ("You need to install numpy and matplotlib!", STATUS_WAIT)
			return False

		inputs = None
		if   self._scatterPlot.isVisible () == True:
			inputs = self._scatterInputs
		elif self._contourPlot.isVisible () == True:
			inputs = self._contourInputs
		elif self._histogramPlot.isVisible () == True:
			inputs = self._histogramInputs

		names = []
		accesses = []
		for holder in inputs:
			if holder._item:
				names.append (holder._item._name)
				accesses.append (holder._item._shellAccess)

		if   self._scatterPlot.isVisible () == True:
		# Scatter Plot (optional errorbar)
			if len (accesses) < 2:
				self._bridge._statusBar.showTimed ("You must have at least two arguments!", STATUS_WAIT)
				return
			style = ", PLOT_STYLE=" + str (self.scatterPlotColor.itemData \
				(self.scatterPlotColor.currentIndex ()).toString ()) + \
				str (self.scatterStyle.itemData (self.scatterStyle.currentIndex ()).toString ())

			if self._xerror.isEnabled () or self._yerror.isEnabled ():
				self._bridge._ipyShell.Command ("plot (" + accesses[0] + ", " + accesses[1] + \
					", " + str (self._yerror.value ()) + ", " + str (self._xerror.value ()) + \
					style + ", \"pylab, errorbar, title=" + \
					self._scatterName.text () + "\")")
			else:
				self._bridge._ipyShell.Command ("plot (" + accesses[0] + ", " + accesses[1] + \
					style + ", \"pylab, scatter, title=" + \
					self._scatterName.text () + "\")")
		elif self._contourPlot.isVisible () == True:
		# Contour Plot
			if   len (accesses) == 1:
				if (self._ContourSwitch.checkState() == PyQt4.QtCore.Qt.Checked): 
					self._bridge._ipyShell.Command ("plot (" + accesses[0] + \
						", \"pylab, contour, title=" + self._contourName.text () + \
						", " + str (self.contourPlotColor.currentText ()) + "\")")
				if (self._ColorSwitch.checkState() == PyQt4.QtCore.Qt.Checked): 
					self._bridge._ipyShell.Command ("plot (" + accesses[0] + \
						", \"pylab, pcolor, title=" + self._contourName.text () + \
						", " + str (self.contourPlotColor.currentText ()) + "\")")	
			elif len (accesses) == 3:
				self._bridge._ipyShell.Command ("plot (" + accesses[0] + ", " + accesses[1] + \
					", " + accesses[2] + ", \"pylab, contour, title=" + self._contourName.text () + \
					", " + str (self.contourPlotColor.currentText ()) + "\")")
			else:
				self._bridge._statusBar.showTimed ("Improper number of arguments (1 or 3)!", STATUS_WAIT)
				return
		elif self._histogramPlot.isVisible () == True:
		# Histogram Plot
			if len (accesses) < 1:
				self._bridge._statusBar.showTimed ("You must have at least one argument!", STATUS_WAIT)
				return
			self._bridge._ipyShell.Command ("plot (" + accesses[0] + ", " + \
				str (self._binsHistogram.value ()) + ", \"pylab, hist, title=" + \
				self._histogramName.text () + "\")")

	def clear (self):
		"""
		PyLab.clear (self)

		This method will empty the input holders as well as
		reset the plot's name to "My Plot".  This will get
		called either explicity when the user clicks on the
		`Clear` button, or on a successful transformation.
		"""
		if   self._scatterPlot.isVisible () == True:
			self._scatterName.setText ("My Scatter Plot")
			for holder in self._scatterInputs:
				holder.clear ()
		elif self._contourPlot.isVisible () == True:
			self._contourName.setText ("My Contour Plot")
			for holder in self._contourInputs:
				holder.clear ()
		elif self._histogramPlot.isVisible () == True:
			self._histogramName.setText ("My Histogram Plot")
			for holder in self._histogramInputs:
				holder.clear ()

class VisItThread (PyQt4.QtCore.QThread):
	def __init__ (self, bridge, parent):
		PyQt4.QtCore.QThread.__init__ (self)

		self._bridge = bridge
		self._visit = None
		self._windowCount = 0
		self._database = None

		# Make this a setting later
		self._plotTimeout = 10

		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadLaunch (PyQt_PyObject)"), self.Launch, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadClose ()"), self.Close, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadAddWindow ()"), self.AddWindow, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadSetActiveWindow (PyQt_PyObject)"), self.SetActiveWindow, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadOpenComputeEngine ()"), self.OpenComputeEngine, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadOpenSimulation ()"), self.OpenSimulation, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadReOpenSimulation ()"), self.ReOpenSimulation, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadCloseSimulation ()"), self.CloseSimulation, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadAddPlotPseudocolor (PyQt_PyObject)"), self.AddPlotPseudocolor, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadAddPlotVolume (PyQt_PyObject)"), self.AddPlotVolume, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadInvertBackgroundColor ()"), self.InvertBackgroundColor, PyQt4.QtCore.Qt.QueuedConnection)
		PyQt4.QtCore.QObject.connect (parent, PyQt4.QtCore.SIGNAL ("threadDrawPlots ()"), self.DrawPlots, PyQt4.QtCore.Qt.QueuedConnection)

	def getWindowCount (self):
		return self._windowCount

	def getDatabase (self):
		# if it doesn't exist yet, we gotta go bug the ipython shell for it...
		# this is mildly costly, so once its found save it to self._database
		if not self._database:
			# Determine the path of the simulation file
			path = self._bridge._ipyShell.ControllerCommand ("print \"\\\\n\" + filesystem.home ()").split ("\n")[-1]
			path = path + "/.visit/simulations"
			# Determine the name of the simulation file
			simulations = self._bridge._ipyShell.ControllerCommand \
				("print str (filesystem.lsFile (\"" + path + "\"))").split ("\n")
			simulations.sort ()
			simulation = simulations[-5]
			simulation = simulation[:simulation.find ("$")]
			self._database = str (self._bridge._host) + ":" + path + "/" + simulation

		# regardless of how you got the sim file database, return it
		return self._database

	def run (self):
		self.exec_ ()

	def launched (self):
		if self._visit:
			return True
		return False

	def Launch (self, stereo):
		if self.launched ():
			return

		print "VisIt Thread: Launching"
		import visit
		if stereo:
			visit.AddArgument ("-stereo")
		visit.Launch ()
		self._windowCount = 1
		self._visit = visit
		del (visit)

	def Close (self):
		if self.launched ():
			print "VisIt Thread: Closing"
			self._visit.Close ()
			del (self._visit)
			self._visit = None
			return

		print "VisIt Thread: Closing (Error, Not Launched)"

	def AddWindow (self):
		if self.launched ():
			self._windowCount += 1
			print "VisIt Thread: Adding Window #" + str (self._windowCount)
			self._visit.AddWindow ()
			return

		print "VisIt Thread: Adding Window (Error, Not Launched)"

	def SetActiveWindow (self, index):
		if index < 1 or index > self._windowCount:
			print "VisIt Thread: Setting Active Window (Error, No Window #" + str (index) + ")"
			return

		if self.launched ():
			print "VisIt Thread: Setting Active Window (" + str (index) + ")"
			self._visit.SetActiveWindow (index)
			return

		print "VisIt Thread: Setting Active Window (Error, Not Launched)"

	def OpenComputeEngine (self):
		if self.launched ():
			print "VisIt Thread: Opening Computation Engine"
			host = str (self._bridge._host)
			args = ("-dir", str (self._bridge._ipyShell.VisItPath ()))
			self._visit.OpenComputeEngine (host, args)
			return

		print "VisIt Thread: Opening Computation Engine (Error, Not Launched)"

	def OpenSimulation (self):
		if self.launched ():
			print "VisIt Thread: Opening Simulation"
			self._visit.OpenDatabase (self.getDatabase ())
			return

		print "VisIt Thread: Opening Simulation (Error, Not Launched)"

	def ReOpenSimulation (self):
		if self.launched ():
			print "VisIt Thread: Reopening Simulation"
			self._visit.ReOpenDatabase (self.getDatabase ())
			return

		print "VisIt Thread: Reopening Simulation (Error, Not Launched)"

	def CloseSimulation (self):
		if self.launched ():
			print "VisIt Thread: Closing Simulation"
			self._visit.CloseDatabase (self.getDatabase ())
			return
		
		print "VisIt Thread: Closing Simulation (Error, Not Launched)"

	def AddPlotPseudocolor (self, name):
		if self.launched ():
			print "VisIt Thread: Adding Pseudocolor Plot"
			count = self._plotTimeout
			while self._visit.AddPlot ("Pseudocolor", name) == 0 and count > 0:
				count -= 1
				print "VisIt Thread: Adding Pseudocolor Plot (Stalled)"
				time.sleep (1)

			if count == 0:
				print "VisIt Thread: Adding Pseudocolor Plot (Failure)"
				return
			return

		print "VisIt Thread: Adding Pseudocolor Plot (Error, Not Launched)"

	def AddPlotVolume (self, name):
		if self.launched ():
			print "VisIt Thread: Adding Volume Plot"
			count = self._plotTimeout
			while self._visit.AddPlot ("Volume", name) == 0 and count > 0:
				count -= 1
				print "VisIt Thread: Adding Volume Plot (Stalled:", count, ")"
				time.sleep (1)

			if count == 0:
				print "VisIt Thread: Adding Volume Plot (Failure)"
				return
			return

		print "VisIt Thread: Adding Volume Plot (Error, Not Launched)"

	def DrawPlots (self):
		if self.launched ():
			print "VisIt Thread: Drawing Plots"
			self._visit.DrawPlots ()
			return

		print "VisIt Thread: Drawing Plots (Error, Not Launched)"

	def InvertBackgroundColor (self):
		if self.launched ():
			print "VisIt Thread: Inverting Background Color"
			self._visit.InvertBackgroundColor ()
			return

		print "VisIt Thread: Inverting Background Color (Error, Not Launched)"

