"""
Overview
========
This module provides the menuing system for the Basin Client.  It is used in conjunction with the Actions
module so that it can take every action from each package and give it its own menu.
"""

import PyQt4.QtGui

import re

import Constants

class MenuBar (PyQt4.QtGui.QMenuBar):
	"""
	Overview
	========
	This class is a convenienve class which bundles the adding of all the menu items into
	the menu's constructor.  The way it can do this is by asking the Action object, for
	which is has access to from the bridge, for all of the actions it has under a given
	package.  This helps in automating the process of creating menus, especially for
	large menus such as "General Functions".
	"""

	def __init__ (self, parent, bridge):
		"""
		Overview
		========
		The constructor will pull all of the actions from each individual package of actions
		from the Actions object.  For each package, a seperatorList is defined as a list of
		names of actions in that package.  For each name in the list, a seperator will appear
		after that action.  In this way, the user doesn't have to worry about anything other
		than how to seperate out actions in a package, the rest is taken care of.

		@param parent: The main window for which this menubar will be applied to.
		@type parent: QMainWindow

		@param bridge: The bridge object which connects all interactive components together.
		@type bridge: L{Bridge<client.Bridge.Bridge>}
		"""

		PyQt4.QtGui.QMenuBar.__init__ (self, parent)

		self._bridge = bridge
		bridge.setMenuBar (self)

		# `seperatorList` is a list defined before each menu is created
		# you should supply the NAMES of each action (i.e. action.text ())
		# and it will generate a separator AFTER each occurance of the name
		# found in the list seperatorList

		self.file = PyQt4.QtGui.QMenu ("&File", parent)
		seperatorList = ["Update", "Save", "Load Batch File"]
		for act in self._bridge._actions._file:
			self.file.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.file.addSeparator ()
		self.addMenu (self.file)

		self.plot = PyQt4.QtGui.QMenu ("&Plotting", parent)
		seperatorList = []
		for act in self._bridge._actions._plot:
			self.plot.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.plot.addSeparator ()
		self.addMenu (self.plot)

		self.genf = PyQt4.QtGui.QMenu ("&General Functions", parent)
		seperatorList = ["Attribute Index", "Imaginary", "Index Sort", "exp10"]
		for act in self._bridge._actions._genf:
			self.genf.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.genf.addSeparator ()
		self.addMenu (self.genf)

		self.cosm = PyQt4.QtGui.QMenu ("&Cosmology", parent)
		seperatorList = ["Redshift To Growth"]
		for act in self._bridge._actions._cosm:
			self.cosm.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.cosm.addSeparator ()
		self.addMenu (self.cosm)

		self.lina = PyQt4.QtGui.QMenu ("&Linear Algebra", parent)
		seperatorList = ["Create Identity Matrix"]
		for act in self._bridge._actions._linalg:
			self.lina.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.lina.addSeparator ()
		self.addMenu (self.lina)

		self.stel = PyQt4.QtGui.QMenu ("Stellar &Dynamics", parent)
		seperatorList = []
		for act in self._bridge._actions._stel:
			self.stel.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.stel.addSeparator ()
		self.addMenu (self.stel)

		self.stat = PyQt4.QtGui.QMenu ("&Statistics", parent)
		seperatorList = []
		for act in self._bridge._actions._stat:
			self.stat.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.stat.addSeparator ()
		self.addMenu (self.stat)

		self.tran = PyQt4.QtGui.QMenu ("&Transformations", parent)
		seperatorList = ["Cartesian To Spherical", "Cartesian To Equatorial", \
			"Cartesian To Polar", "Cartesian To Cylindrical", "FFT", "Convolve"]
		for act in self._bridge._actions._tran:
			self.tran.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.tran.addSeparator ()
		self.addMenu (self.tran)

		self.help = PyQt4.QtGui.QMenu ("&Help", parent)
		seperatorList = []
		for act in self._bridge._actions._help:
			self.help.addAction (act)
			if seperatorList.count (act.text ()) > 0: 
				self.help.addSeparator ()
		self.addMenu (self.help)

		self._bridge._actions.menuDisconnect ()
		parent.setMenuBar (self)
		self._bridge._toolBar.loadSettings ()
