from PyQt4.QtGui  import *
from PyQt4.QtCore import *

import re
import Constants

# This class, when created, will go through everything until execution.
# The user will see a window of files and folders, and choose a file eventually.
# After the user accepts or rejects this dialog, the final result, if any, is stored
# is self.finalresult, which may be accessed.  Note that this class merely chooses a
# file from a remote location, it does nothing with that file name.

# The way it accomplishes this is through the basin_remote on the host side.
# A function was written to reveal the directory it keeps scripts that this client
# relies on.  Those scripts were improted, and kept the name `filesystem`, which
# has three functions:
#	filesystem.home (): 
#		returns the remote user's home
#	filesystem.lsDir (path = home ()):
#		Given the path, or default home, it will display all
#		directories in the path
#	filesystem.lsFile (path = home ()):
#		Given the path, or default home, it will display all
#		files in the path
class RemoteFileBrowser (QDialog):
	def __init__ (self, bridge):
		# Setup the dialog to be half the size of the main window, and centered over it
		QDialog.__init__ (self, bridge._mainWindow)
		self._bridge = bridge
		mainRect = self._bridge._mainWindow.geometry  ()
		width = mainRect.width ()
		height = mainRect.height ()
		x = mainRect.x () + int (width * 0.175)
		y = mainRect.y () + int (height * 0.25)
		width = int (width * 0.65)
		height = int (height * 0.5)
		self.setGeometry (x, y, width, height)

		# The remote file browser bases its remote-ness off of a script that it finds
		# on the remote server.  The script is imported to the controller, not the
		# cluster, thus everything MUST be done locally
		self._bridge.makeLocal ()

		# Where the directory and file items will be held for each path.
		# They are stored because the user may wish to change the file types
		# they browse, and I would rather store the total ls result than
		# continually probe the controller.
		self.dirs = []
		self.files = []

		# will be accessed after the dialog is executed, for the file chosen
		# it will contain the full file path and file name
		self.finalresult = []

		# This is just dialog box stuff, a lot of it has been extracted into
		# separate functions for the beauty of the code, but for no great purpose
		self.layout = QVBoxLayout (self)
		self.addPath ()
		self.list = RemoteFileList (self.layout, self)
		self.addFileInfo ()
		self.addButtons ()
		self.WindowStuff ()

		if self._bridge._dataPath:
			self.path.setPath (self._bridge._dataPath)
		else:
			self.Home ()
		self.ls ()

		# apply the layout to the dialog, then execute the dialog
		# meaning that it will be displayed until resolved
		self.exec_ ()

	# Sets the current path to the user's home, then ls's it
	def Home (self):
		self._bridge.CursorBusy ()
		self.path.setPath (self._bridge._ipyShell.ControllerCommand \
			("print \"\\\\n\" + filesystem.home ()").split ("\n")[-1])
		self._bridge.CursorNorm ()
		self.ls ()

	# Populates the lisst self.firs and self.files, then sends them to be shown
	# based on the type of file the user currently wishes to view
	def ls (self):
		outRegexp = re.compile ("\[.*\]\s*Out\s*\[.*\]:")
		self._bridge.CursorBusy ()
		# Gets the correct path (note that the path doesn't have to be valid)
		path = str (self.path.text ())

		hidden = ""
		if self.hiddenCheck.checkState () == Qt.Checked:
			hidden = ", True"

		# ipython command, local, to get directories
		result = self._bridge._ipyShell.ControllerCommand \
			("print str (filesystem.lsDir (\"" + path + "\"" + hidden + "))")
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_remote_file.py", "RemoteFileBrowser", "ls", result)
			return
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i

		self.dirs = []
		for each in result[index + 1:]:
			self.dirs.append (Path (path, each.split ("$")))

		# ipython command, local, to get all files
		result = self._bridge._ipyShell.ControllerCommand \
			("print str (filesystem.lsFile (\"" + path + "\"" + hidden + "))")
		if result.find ("rror") != -1:
			self._bridge.Error ("BASIN_remote_file.py", "RemoteFileBrowser", "ls", result)
			return
		result = result.split ("\n")
		index = 0
		for i in xrange (len (result)):
			if outRegexp.match (result[i]):
				index = i

		self.files = []
		for each in result[index + 1:]:
			self.files.append (Path (path, each.split ("$")))

		# sort the files alphabetically
		# self.files.sort ()

		# function that will display them
		self.showType ()
		self._bridge.CursorNorm ()

	def goActivated (self):
		self.path.setPath (self.path.text ())
		self.ls ()

	# Shows all directories, and the files that meet the regular expression
	# of the type the user chose (or defaulted to) in the combobox
	def showType (self):
		# wipe out the current list
		self.Clear ()
		self.list.setSortingEnabled (False)

		index = self.filetype.currentIndex ()
		variant = self.filetype.itemData (index)
		regexp = re.compile (str (variant.toString ()))
		tempFiles = []
		for file in self.files:
			if regexp.match (str (file._name)):
				tempFiles.append (file)

		self.list.setRowCount (len (self.dirs) + len (tempFiles))

		row = 0
		for dir in self.dirs:
			self.addPathToList (dir, row, True)
			row += 1
		for file in tempFiles:
			self.list.setRowCount (row + 1)
			self.addPathToList (file, row, False)
			row += 1

		# Sort alphabetically by name, then by file type
		self.list.setSortingEnabled (True)
		self.list.resizeColumnsToContents ()
		self.list.sortItems (0)
		self.list.sortItems (1)

	# Just sets the title bar and icon of the window, overloaded for exporting
	def WindowStuff (self):
		self.setWindowTitle ("Opening from " + str (self._bridge._host) +
			":" + str (self._bridge._port))
		self.setWindowIcon (QIcon (QString (Constants.ICON_DIR + "/file/open.png")))

	# adds the path, and path buttons like `Go` and `Up`
	def addPath (self):
		pathLayout = Constants.BQHBoxLayout (self.layout)
		self.path = RemotePathBar (pathLayout, self)
		self.pathButton = QPushButton ("Go")
		self.homeButton = QPushButton (QIcon (Constants.ICON_DIR + "/misc/home.png"), "")
		self.upButton = QPushButton (QIcon (Constants.ICON_DIR + "/misc/arrowup.png"), "")
		pathLayout.addWidget (self.pathButton)
		pathLayout.addWidget (self.homeButton)
		pathLayout.addWidget (self.upButton)
		QObject.connect (self.pathButton, SIGNAL ("clicked ()"), self.goActivated)
		QObject.connect (self.homeButton, SIGNAL ("clicked ()"), self.Home)
		QObject.connect (self.upButton, SIGNAL ("clicked ()"), self.path.up)

	# adds the buttons import and cancel to the dialog layout
	def addButtons (self):
		buttonLayout = Constants.BQHBoxLayout (self.layout)
		self.hiddenCheck = QCheckBox ("Show Hidden")
		QObject.connect (self.hiddenCheck, SIGNAL ("stateChanged (int)"), self.ls)
		buttonLayout.addWidget (self.hiddenCheck)

		# Make a combo box of file types and their corresponding regular expression
		self.filetype = QComboBox ()
		QObject.connect (self.filetype, SIGNAL ("currentIndexChanged (int)"), self.showType)
		typeLabel = QLabel ("File Type:")
		typeLabel.setAlignment (Qt.AlignRight | Qt.AlignVCenter)
		buttonLayout.addWidget (typeLabel)
		buttonLayout.addWidget (self.filetype)
		self.filetype.addItem (QString ("All Files (*.*)"), QVariant (QString (".*")))
		self.filetype.addItem (QString ("BASIN (*.dat, *.txt)"), QVariant (QString (".*\.(dat|txt)")))
		self.filetype.addItem (QString ("HDF5 (*.h5)"), QVariant (QString (".*\.h5")))
		self.filetype.addItem (QString ("FITS (*.fits, *.fit)"), QVariant (QString (".*\.fits?")))
		self.filetype.addItem (QString ("StarLab (*.strlab)"), QVariant (QString (".*\.strlab")))

		self.actionButton = QPushButton (QIcon (Constants.ICON_DIR + "/file/open.png"), QString ("Open"))
		QObject.connect (self.actionButton, SIGNAL ("clicked ()"), self.accept)
		buttonLayout.addWidget (self.actionButton)
		self.cancelButton = QPushButton (QString ("Cancel"))
		QObject.connect (self.cancelButton, SIGNAL ("clicked ()"), self.reject)
		buttonLayout.addWidget (self.cancelButton)

	# Adds the lineedit and combobox to the dialog layout
	def addFileInfo (self):
		self.filename = QLineEdit (self)
		QObject.connect (self.filename, SIGNAL ("returnPressed ()"), self.accept)
		self.layout.addWidget (self.filename)

	# Overwrites QDialog's accept function so that
	# %autopx is restored, and the finalresult is set
	# noting that if no file is selected, accept does nothing
	def accept (self):
		items = self.list.selectedItems ()
		if   (self.filename.text ()) != "":
			self.finalresult.append (str (self.path._path) + "/" + str (self.filename.text ()))
		elif len (items) == 1 and items[0] != None and items[0].Type () == "FILE":
			self.finalresult.append (items[0]._path)
		self._bridge.makeParallel ()
		QDialog.accept (self)
	
	# Overwrites QDialog's reject function so that
	# %autopx is restored
	def reject (self):
		self._bridge.makeParallel ()
		QDialog.reject (self)

	# Clear off the file/dir list,
	# generally speaking to make room for another
	def Clear (self):
		self.list.clearContents ()

	# Creates the appropriate Item for the list for Files
	def addPathToList (self, file, row, isFolder):
		provider = QFileIconProvider ()
		if isFolder:
			_icon = provider.icon (QFileIconProvider.Folder)
			_type = "Folder"
		else:
			_icon = provider.icon (QFileIconProvider.File)
			_type = "file"
			if   file._name[-3:] == ".h5":
				_type = "hdf5"
			elif file._name[-4:] == ".fit":
				_type = "fits"
			elif file._name[-5:] == ".fits":
				_type = "fits"
			elif file._name[-4:] == ".dat":
				_type = "basin"
			elif file._name[-4:] == ".txt":
				_type = "basin"
			elif file._name[-7:] == ".strlab":
				_type = "starlab"

		cellHeight = self.list.rowHeight (0)
		gradient = QLinearGradient (QPointF (0, 0), QPointF (0, cellHeight))
		gradient.setColorAt (0.0, QColor (200, 250, 200))
		gradient.setColorAt (0.5, Qt.white)
		gradient.setColorAt (1.0, QColor (200, 250, 200))
		brush = QBrush (gradient)

		name = QTableWidgetItem (_icon, file._name)
		name.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		name.setTextColor (Qt.black)
		name.setTextAlignment (Qt.AlignVCenter | Qt.AlignLeft)
		name.setBackground (brush)
		self.list.setItem (row, 0, name)

		type = QTableWidgetItem (_type)
		type.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		type.setTextColor (Qt.darkGray)
		type.setTextAlignment (Qt.AlignVCenter | Qt.AlignRight)
		type.setBackground (brush)
		self.list.setItem (row, 1, type)

		user = QTableWidgetItem (file._user)
		user.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		user.setTextColor (Qt.darkGray)
		user.setTextAlignment (Qt.AlignVCenter | Qt.AlignRight)
		user.setBackground (brush)
		self.list.setItem (row, 2, user)

		group = QTableWidgetItem (file._group)
		group.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		group.setTextColor (Qt.darkGray)
		group.setTextAlignment (Qt.AlignVCenter | Qt.AlignRight)
		group.setBackground (brush)
		self.list.setItem (row, 3, group)

		size = QTableWidgetItem (file._size)
		size.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		size.setTextColor (Qt.darkGray)
		size.setTextAlignment (Qt.AlignVCenter | Qt.AlignRight)
		size.setBackground (brush)
		self.list.setItem (row, 4, size)

		aTime = QTableWidgetItem (file._aTime)
		aTime.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		aTime.setTextColor (Qt.darkGray)
		aTime.setTextAlignment (Qt.AlignVCenter | Qt.AlignRight)
		aTime.setBackground (brush)
		self.list.setItem (row, 5, aTime)

		mTime = QTableWidgetItem (file._mTime)
		mTime.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		mTime.setTextColor (Qt.darkGray)
		mTime.setTextAlignment (Qt.AlignVCenter | Qt.AlignRight)
		mTime.setBackground (brush)
		self.list.setItem (row, 6, mTime)

		cTime = QTableWidgetItem (file._cTime)
		cTime.setFlags (Qt.ItemIsSelectable | Qt.ItemIsEnabled)
		cTime.setTextColor (Qt.darkGray)
		cTime.setTextAlignment (Qt.AlignVCenter | Qt.AlignRight)
		cTime.setBackground (brush)
		self.list.setItem (row, 7, cTime)


# The same basic principle as before, however a few changes were
# made to have it support exporting instead of importing
class RemoteFileBrowserExport (RemoteFileBrowser):
	def __init__ (self, bridge):
		RemoteFileBrowser.__init__ (self, bridge)
	
	# different icon and window title for exporting done here
	def WindowStuff (self):
		self.setWindowTitle ("Saving to " + str (self._bridge._host) + \
			":" + str (self._bridge._port))
		self.setWindowIcon (QIcon (QString (Constants.ICON_DIR + "/file/save.png")))

	# different buttons are created here for exporting
	def addButtons (self):
		buttonLayout = Constants.BQHBoxLayout (self.layout)
		self.hiddenCheck = QCheckBox ("Show Hidden")
		QObject.connect (self.hiddenCheck, SIGNAL ("stateChanged (int)"), self.ls)
		buttonLayout.addWidget (self.hiddenCheck, Qt.AlignHCenter)

		# Make a combo box of file types and their corresponding regular expression
		self.filetype = QComboBox ()
		QObject.connect (self.filetype, SIGNAL ("currentIndexChanged (int)"), self.showType)
		typeLabel = QLabel ("File Type:")
		typeLabel.setAlignment (Qt.AlignRight | Qt.AlignVCenter)
		buttonLayout.addWidget (typeLabel)
		buttonLayout.addWidget (self.filetype)
		self.filetype.addItem (QString ("All Files (*.*)"), QVariant (QString (".*")))
		self.filetype.addItem (QString ("BASIN (*.dat, *.txt)"), QVariant (QString (".*\.(dat|txt)")))
		self.filetype.addItem (QString ("HDF5 (*.h5)"), QVariant (QString (".*\.h5")))
		self.filetype.addItem (QString ("FITS (*.fits, *.fit)"), QVariant (QString (".*\.fits?")))
		self.filetype.addItem (QString ("StarLab (*.strlab)"), QVariant (QString (".*\.strlab")))

		self.actionButton = QPushButton (QIcon (Constants.ICON_DIR + "/file/save.png"), QString ("Save"))
		QObject.connect (self.actionButton, SIGNAL ("clicked ()"), self.accept)
		buttonLayout.addWidget (self.actionButton, Qt.AlignHCenter)
		self.cancelButton = QPushButton (QString ("Cancel"))
		QObject.connect (self.cancelButton, SIGNAL ("clicked ()"), self.reject)
		buttonLayout.addWidget (self.cancelButton, Qt.AlignHCenter)

	# accept is much different from importing in that the file type you are browsing
	# will append its extension onto your given filename (which doesn't need to exist)
	# if it doesn't detect it being there already.
	def accept (self):
		self._bridge.makeParallel ()
		item = str (self.path._path)

		# if it doesn't have a leading "/", most likely the case, add it
		if item[len (item) - 1] != "/":
			item += "/"
		# then tack on the file name you are accepting
		item += str (self.filename.text ())
		if item.strip () != "":
			# get the regular expression corresponding to the file type in the combobox,
			# and if it doesn't meet the expression, then add the correct extension.
			index = self.filetype.currentIndex ()
			variant = self.filetype.itemData (index)
			if not re.compile (str (variant.toString ())).match (str (item)):
				type = str (self.filetype.currentText ())
				if type.find == "HDF5 (*.h5)":
					item += ".h5"
				elif type == "BASIN (*.dat, *.txt)":
					item += ".dat"
				elif type == "FITS (*.fits)":
					item += ".fits"
				elif type == "StarLab (*.strlab)":
					item += ".strlab"
			# set it as the final result, and finally accept
			self.finalresult = [item]
			QDialog.accept (self)

# This provides a bit more convenience than a normal QLineEdit because of the builtin function self.up
# which essentially just chops off the last directory from the path, then recalls ls from its parent,
# the RemoteFileBrowser(Export).  This isn't so much needed as nice to have.
class RemotePathBar (QLineEdit):
	def __init__ (self, layout, parent):
		QLineEdit.__init__ (self)
		label = QLabel ("Path:")
		layout.addWidget (label)
		layout.addWidget (self)
		self._parent = parent
		self._path = ""

	def setPath (self, path):
		self._path = path
		self.setText (path)

	# Exactly what Up ought to do when traversing files/directories...
	# Chops the last chunk fo the path off, then goes to that new, shorter, path
	def up (self):
		text = self._path
		if text == "/":
			return
		if text[len (text) - 1] == "/":
			text = text[:-1]
		while text[len (text) - 1] != "/":
			text = text[:-1]
		if text != "/":
			text = text[:-1]
		self.setPath (text)
		self._parent.ls ()

# This is the place where all of the actual directories and files are listed.
# This HAD to be a derived QListWidget because the clicking mechanisms needed
# to be overloaded.  Other than that, in every regard it is merely a QListWidget
class RemoteFileList (QTableWidget):
	def __init__ (self, layout, parent):
		QTableWidget.__init__ (self)
		self.setShowGrid (False)
		self.setSelectionBehavior (QAbstractItemView.SelectRows)
		self.setSelectionMode (QAbstractItemView.SingleSelection)
		self.verticalHeader ().hide ()
		labels = ["Name", "Type", "User", "Group", "Size", "Last Access", "Last Mod", "Created On"]
		self.setColumnCount (len (labels))
		self.setHorizontalHeaderLabels (labels)
		layout.addWidget (self)
		self._parent = parent
		QObject.connect (self, SIGNAL ("cellDoubleClicked (int,int)"), self.DoubleClick)
		QObject.connect (self, SIGNAL ("currentCellChanged (int,int,int,int)"), self.CellChanged)

	# On a single click (note that is is also in a double click), regardless of the mouse button used,
	# if you clicked on a file, it goes to the bar UNDER the this object, the list of files/directories,
	# and over the combo box.  This is needed mostly for exporting, which requires there to be a file
	# in  that bar, simply because you may be creating a file that exists nowhere else.
	def SingleClick (self, row, col):
		if self.item (row, 1):
			type = self.item (row, 1).text ()
		else:
			return

		if type == "Folder":
			self._parent.filename.setText ("")
		else:
			item = self.item (row, 0).text ()
			self._parent.filename.setText (item)

	def CellChanged (self, newRow, newCol, prevRow, prevCol):
		if newRow == -1:
			newRow = 0
		self.SingleClick (newRow, newCol)

	def keyPressEvent (self, ev):
		key = ev.key ()
		if   key == Qt.Key_Enter or key == Qt.Key_Return:
			self.DoubleClick (self.currentRow ())
		elif key == Qt.Key_Up:
			self.selectRow (self.currentRow () - 1)
		elif key == Qt.Key_Down:
			self.selectRow (self.currentRow () + 1)

	# Note again, that a singleclick must have occurred to get here...
	# If you double click a folder, you add that folder to the path, and
	# begin again by calling ls ().  If you double clicked a file, then you
	# just called accept, and because a single click happenned, either a filename
	# sits to be accepted, or nothing because folders don't add themselves to the file bar
	def DoubleClick (self, row, col = 0):
		type = self.item (row, 1).text ()
		if type == "Folder":
			item = self.item (row, 0).text ()
			if self._parent.path._path == "/":
				self._parent.path.setPath (self._parent.path._path + item)
			else:
				self._parent.path.setPath (self._parent.path._path + "/" + item)
			self._parent.ls ()
		else:
			self._parent.accept ()


class Path:
	def __init__ (self, path, val):
		length = len (val)

		if length >= 0:
			self._name = val[0]
		self._fullPath = path + "/" + self._name

		if length >= 1:
			self._user = val[1]

		if length >= 2:
			self._group = val[2]

		if length >= 3:
			self._size = val[3]

		if length >= 4:
			self._aTime = val[4]

		if length >= 5:
			self._mTime = val[5]

		if length >= 6:
			self._cTime = val[6]
