"""
Overview
========
This modules provides a Status Bar for the Basin Client Interface.  It has a much simpler
interface than its parent class, QStatusBar, since it needs to do far less.
"""

import PyQt4.QtGui

class StatusBar (PyQt4.QtGui.QStatusBar):
	"""
	Overview
	========
	This class acts as a mere facade for the PyQt status bar interface.  All we need it to ever do is show text
	with or without a timer on them.
	"""

	def __init__ (self, parent, bridge):
		"""
		Overview
		========
		This constructor will set itself (a QStatusBar) to the parent, and will connect itself to the 
		bridge.  This object will essentially just sit and wait for messages to be sent to it.

		@param parent: The main window which the status bar will be inserted into.
		@type parent: QMainWindow

		@param bridge: The bridge object which connects all interactive components together.
		@type bridge: L{Bridge<client.Bridge.Bridge>}
		"""

		PyQt4.QtGui.QStatusBar.__init__ (self, parent)

		# Connect to bridge & parent
		bridge.setStatusBar (self)
		parent.setStatusBar (self)

	def show (self, text):
		"""
		Overview
		========
		This will show a message on the status bar for an indefinite amount of time.  As soon as this
		method is called again, or the showTimed method, the message in 'text' will be taken down and
		replaced with the new message.

		@param text: The message you want displayed in the status bar.
		@type text: string
		"""

		self.showMessage (str (text))

	def showTimed (self, text, seconds):
		"""
		Overview
		========
		This will show a message on the status bar for a specified time in seconds (a float).  The
		message in 'text' may come down sooner, if this method, or the show method is called before the
		time has elapsed.

		@param text: The message you want displayed in the status bar.
		@type text: string

		@param seconds: The number of seconds you want the message to be displayed for.
		@type seconds: int or float
		"""

		self.showMessage (str (text), seconds * 1000.0)
